// =============================================================================
// Module com_ram_tdp_clkgate_ab
// Revision 20170109a : Initial revision
// Clock selector.
// =============================================================================

`timescale 1ps/1ps

module com_ram_tdp_clkgate_ab (
    clk_a,
    clk_b,
    clk_a_en,
    clk_b_en,
    clk_out
);
    input  clk_a;
    input  clk_b;
    input  clk_a_en;
    input  clk_b_en;
    output clk_out;

    reg    reg_clk_a_en;
    reg    reg_clk_b_en;

    wire clk_a_out;
    wire clk_b_out;

    /* ========================================================================
       Caution : Not glitch-less selector !!
       ========================================================================
               reg_clk_a_en
                   +---+
       clk_a_en--->|D Q|--->+---+  clk_a_out
       clk_a---+->o|G  |    |AND|------------->+--+
               |   +---+ +->+---+              |OR|--->clk_out
               |         |        +----------->+--+
               +---------+        |
                                  |
               reg_clk_b_en       |clk_b_out
                   +---+          |
       clk_b_en--->|D Q|--->+---+ |
       clk_b---+->o|G  |    |AND|-+
               |   +---+ +->+---+
               |         |
               +---------+
       ======================================================================== */

    // Delay isn't added for clock path.
    always @(clk_a or clk_a_en) begin
        if (~clk_a) begin
            reg_clk_a_en <= clk_a_en;
        end
    end

    // Delay isn't added for clock path.
    always @(clk_b or clk_b_en) begin
        if (~clk_b) begin
            reg_clk_b_en <= clk_b_en;
        end
    end

    assign clk_a_out = clk_a & reg_clk_a_en;
    assign clk_b_out = clk_b & reg_clk_b_en;

    assign clk_out = clk_a_out | clk_b_out;

endmodule
